#	$Id: bt.lib.mk,v 1.36 2003/08/30 10:06:57 jmmv Exp $
#	$NetBSD: bsd.lib.mk,v 1.205 2002/07/20 08:50:10 yamt Exp $
#	@(#)bsd.lib.mk	8.3 (Berkeley) 4/22/94

.include <bt.init.mk>

.SUFFIXES: .c .cc .cpp .o .h .sh .m4 .y .s .S .so .a .po

# --------------------------------------------------------------------
# Basic targets
# --------------------------------------------------------------------

BT_LIB_INSTALL?=	YES
.if !empty(BT_LIB_INSTALL:M[Yy][Ee][Ss])
.PHONY:		_libinstall _libdeinstall
real-install:	_libinstall
real-deinstall:	_libdeinstall
.endif

.if !defined(BT_LIB_MAJOR) || empty(BT_LIB_MAJOR)
.abort Version number (at least major) required to build a library
.endif

# Set ${BT_LIB_VERSION} to something useful.
.if !defined(BT_LIB_VERSION)
.if defined(BT_LIB_MAJOR) && !empty(BT_LIB_MAJOR)
.if defined(BT_LIB_MINOR) && !empty(BT_LIB_MINOR)
.if defined(BT_LIB_MICRO) && !empty(BT_LIB_MICRO)
BT_LIB_VERSION=${BT_LIB_MAJOR}.${BT_LIB_MINOR}.${BT_LIB_MICRO}
.else
BT_LIB_VERSION=${BT_LIB_MAJOR}.${BT_LIB_MINOR}
.endif
.else
BT_LIB_VERSION=${BT_LIB_MAJOR}
.endif
.endif
.endif

_LIBNAME=	lib${BT_LIB}.btl

# --------------------------------------------------------------------
# Implicit conversions
# --------------------------------------------------------------------

# Archive objects

.c.o .c.po:
	@${BT_WRAP} -m compile cc ${BT_FLAGS_CPP} ${BT_FLAGS_CC} \
	    -c ${.IMPSRC} -o ${.TARGET}

.cc.o .cc.po:
	@${BT_WRAP} -m compile c++ ${BT_FLAGS_CPP} ${BT_FLAGS_CXX} \
	    -c ${.IMPSRC} -o ${.TARGET}

.cpp.o .cpp.po:
	@${BT_WRAP} -m compile c++ ${BT_FLAGS_CPP} ${BT_FLAGS_CXX} \
	    -c ${.IMPSRC} -o ${.TARGET}


# --------------------------------------------------------------------
# Main targets
# --------------------------------------------------------------------

# Default sources if not defined.
.if defined(BT_LIB)
BT_SRCS?=	${BT_LIB}.c
.endif

# Determine object files.
.if !empty(BT_SRCS:N*.h)
BT_OBJS+=	${BT_SRCS:N*.h:R:S/$/.o/g}
BT_POBJS+=	${BT_SRCS:N*.h:R:S/$/.po/g}
.endif

# Determine which compiler to use as linker.
.if !empty(BT_SRCS:M*.cc) || !empty(BT_SRCS:M*.cpp)
_CMD_LINK=	c++
.else
_CMD_LINK=	cc
.endif # !empty(BT_SRCS:M*.cc) || !empty(BT_SRCS:M*.cpp)

.if defined(BT_OBJS) && !empty(BT_OBJS)
.NOPATH: ${BT_OBJS} ${_LIBNAME} ${BT_SRCS:C/\..$/.c/}

_OBJS=
.if ${BT_LIB_MKSTATIC} == "yes"
_OBJS+=		${BT_OBJS}
.endif
.if ${BT_LIB_MKPIC} == "yes"
.if ${BT_LIB_MKSTATIC} != "yes"
.for f in ${BT_POBJS}
${f}:		${f:S/po$/o/}
${f:S/po$/o/}:
	@touch ${.TARGET}
.endfor
.endif
_OBJS+=		${BT_POBJS}
.endif
${_LIBNAME}: ${_OBJS}
	@${BT_WRAP} -m link ${_CMD_LINK} ${BT_FLAGS_LD} \
		-o ${_LIBNAME} ${BT_OBJS} ${BT_LIBS} \
		-version-info ${BT_LIB_VERSION}
.endif

real-all: depend ${_LIBNAME}

# --------------------------------------------------------------------
# Installation
# --------------------------------------------------------------------

_libinstall::	${BT_DIR_LIB}/${_LIBNAME}
.PRECIOUS:	${BT_DIR_LIB}/${_LIBNAME}
${BT_DIR_LIB}/${_LIBNAME}! ${_LIBNAME}
.if !exists(${BT_DIR_LIB})
	@${BT_INSTALL_DIR} ${BT_DIR_LIB}
.endif
	@${BT_INSTALL_LIB} ${_LIBNAME} ${BT_DIR_LIB}

# --------------------------------------------------------------------
# Deinstallation
# --------------------------------------------------------------------

.if !target(_libdeinstall)
_libdeinstall::
.if exists(${BT_DIR_LIB}/${_NAMESTATIC})
	rm -f ${BT_DIR_LIB}/${_NAMESTATIC}
.endif
.if exists(${BT_DIR_LIB}/${_NAMEPIC})
	rm -f ${BT_DIR_LIB}/${_NAMEPIC}
.endif
.if exists(${BT_DIR_LIB}/${_NAMEPIC_MAJOR})
	rm -f ${BT_DIR_LIB}/${_NAMEPIC_MAJOR}
.endif
.endif


BT_FILES_CLEAN+= a.out [Ee]rrs mklog core *.core
BT_FILES_CLEAN+= ${_LIBNAME}
BT_FILES_CLEAN+= ${_LIBNAME:S/.btl$/.a/}
BT_FILES_CLEAN+= ${_LIBNAME:S/.btl$/.so*/}
BT_FILES_CLEAN+= ${BT_OBJS} ${BT_POBJS}

.if defined(BT_LIB) && !empty(BT_LIB)
real-tologic: lib-tologic
lib-tologic:
	@echo "logic() {" >> Logic.bt
	@echo "    bt_target ${BT_LIB}" >> Logic.bt
	@echo "}" >> Logic.bt
	@echo >> Logic.bt
	@echo "target_${BT_LIB}() {" >> Logic.bt
	@echo "    BT_TYPE=library" >> Logic.bt
	@echo "    BT_SOURCES=\"${BT_SRCS}\"" >> Logic.bt
.if defined(BT_FLAGS_CPP) && !empty(BT_FLAGS_CPP)
	@echo "    BT_FLAGS_CPP=\"$$BT_FLAGS_CPP ${BT_FLAGS_CPP:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_FLAGS_CC) && !empty(BT_FLAGS_CC)
	@echo "    BT_FLAGS_CC=\"$$BT_FLAGS_CC ${BT_FLAGS_CC:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_FLAGS_CXX) && !empty(BT_FLAGS_CXX)
	@echo "    BT_FLAGS_CXX=\"$$BT_FLAGS_CXX ${BT_FLAGS_CXX:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_FLAGS_LD) && !empty(BT_FLAGS_LD)
	@echo "    BT_FLAGS_LD=\"$$BT_FLAGS_LD ${BT_FLAGS_LD:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_LIBS) && !empty(BT_LIBS)
	@echo "    BT_LIBS=\"$$BT_LIBS ${BT_LIBS:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_MAN) && !empty(BT_MAN)
	@echo "    BT_MANPAGES=\"${BT_MAN}\"" >> Logic.bt
.endif
	@echo "    BT_LIB_MAJOR=${BT_LIB_MAJOR}" >> Logic.bt
	@echo "    BT_LIB_MINOR=${BT_LIB_MINOR}" >> Logic.bt
	@echo "    BT_LIB_MICRO=${BT_LIB_MICRO}" >> Logic.bt
.if defined(BT_INCS) && !empty(BT_INCS)
	@echo "    BT_INCLUDES=\"${BT_INCS}\"" >> Logic.bt
.if defined(BT_INCSDIR) && !empty(BT_INCSDIR)
	@echo "    BT_INCLUDESDIR=\"\$$BT_DIR_INCLUDE/${BT_INCSDIR}\"" >> Logic.bt
.endif
.for _F in ${BT_INCS:O:u}
	@_FDIR=${BT_INCSDIR_${_F:C,/,_,g}:Uunset}; \
	[ "$$_FDIR" != "unset" ] && echo "    BT_INCLUDESDIR_${_F:S|.|_|g}=\"\$$BT_DIR_INCLUDE/$$_FDIR\"" >> Logic.bt || true
.endfor
.endif
	@echo "}" >> Logic.bt
.endif

.include <bt.man.mk>
.include <bt.dep.mk>
.include <bt.clean.mk>

${TARGETS}:	# ensure existence
