#	$Id: bt.prog.mk,v 1.21 2003/09/02 13:43:06 jmmv Exp $
#	$NetBSD: bsd.prog.mk,v 1.156 2002/08/09 00:21:22 thorpej Exp $
#	@(#)bsd.prog.mk	8.2 (Berkeley) 4/2/94

.include <bt.init.mk>

.SUFFIXES: .c .cc .cpp .o .po .h .sh .m4 .y .s .S

# --------------------------------------------------------------------
# Basic targets
# --------------------------------------------------------------------

BT_PROG_INSTALL?=	YES
.if !empty(BT_PROG_INSTALL:M[Yy][Ee][Ss])
.PHONY:		_proginstall _progdeinstall
real-install:	_proginstall
real-deinstall:	_progdeinstall
.endif

# --------------------------------------------------------------------
# Implicit conversions
# --------------------------------------------------------------------

.c.o .c.po:
	@${BT_WRAP} -m compile cc ${BT_FLAGS_CPP} ${BT_FLAGS_CC} \
	            -c ${.IMPSRC} -o ${.TARGET}

.cc.o .cc.po:
	@${BT_WRAP} -m compile c++ ${BT_FLAGS_CPP} ${BT_FLAGS_CXX} \
	            -c ${.IMPSRC} -o ${.TARGET}

.cpp.o .cpp.po:
	@${BT_WRAP} -m compile c++ ${BT_FLAGS_CPP} ${BT_FLAGS_CXX} \
	            -c ${.IMPSRC} -o ${.TARGET}

# --------------------------------------------------------------------
# Main targets
# --------------------------------------------------------------------

# Set default sources, if appropiate.
.if defined(BT_PROG)
BT_SRCS?=	${BT_PROG}.c
BT_MAN?=	${BT_PROG}.1
.endif

# Determine object files.
.if defined(BT_SRCS) && !empty(BT_SRCS:N*.h)
BT_OBJS+=	${BT_SRCS:N*.h:R:S/$/.o/g}
BT_POBJS+=	${BT_SRCS:N*.h:R:S/$/.po/g}
.endif

# If we have object files, set program's build.
.if defined(BT_OBJS) && !empty(BT_OBJS)
.NOPATH: ${BT_OBJS} ${BT_POBJS} ${BT_PROG} ${BT_SRCS:C/\..$/.c/}

# The program target itself. Set a default link if not already
# defined.
_OBJS=
.if ${BT_LIB_MKSTATIC} == "yes"
_OBJS+=		${BT_OBJS}
.endif
.if ${BT_LIB_MKPIC} == "yes"
.if ${BT_LIB_MKSTATIC} != "yes"
.for f in ${BT_POBJS}
${f}:		${f:S/po$/o/}
${f:S/po$/o/}:
	@touch ${.TARGET}
.endfor
.endif
_OBJS+=		${BT_POBJS}
.endif
${BT_PROG}: ${_OBJS}
.if !commands(${BT_PROG})
.if !empty(BT_SRCS:M*.cc) || !empty(BT_SRCS:M*.cpp)
	@${BT_WRAP} -m link c++ ${BT_FLAGS_LD} -o ${.TARGET} \
	            ${BT_OBJS} ${BT_LIBS}
.else
	@${BT_WRAP} -m link cc ${BT_FLAGS_LD} -o ${.TARGET} \
	            ${BT_OBJS} ${BT_LIBS}
.endif
.endif

.endif	# defined(BT_OBJS) && !empty(BT_OBJS)

real-all: depend ${BT_PROG}

BT_FILES_CLEAN+= a.out [Ee]rrs mklog core *.core
BT_FILES_CLEAN+= ${BT_PROG} ${BT_OBJS} ${BT_POBJS}

# --------------------------------------------------------------------
# Installation
# --------------------------------------------------------------------

# Only install if possible.
.if defined(BT_PROG) && !target(_proginstall)

_proginstall:: ${BT_DIR_BIN}/${BT_PROG}
.PRECIOUS: ${BT_DIR_BIN}/${BT_PROG}

__proginstall: .USE
.if !exists(${BT_DIR_BIN})
	@${BT_INSTALL_DIR} ${BT_DIR_BIN}
.endif
	@${BT_INSTALL_BIN} ${.ALLSRC} ${.TARGET}

${BT_DIR_BIN}/${BT_PROG}! ${BT_PROG} __proginstall

.endif # defined(BT_PROG) && !target(_proginstall)

# Define an emtpy target, if needed.
.if !target(_proginstall)
_proginstall::
.endif

# --------------------------------------------------------------------
# Deinstallation
# --------------------------------------------------------------------

.if defined(BT_PROG) && !target(_progdeinstall)
_progdeinstall::
.if exists(${BT_DIR_BIN}/${BT_PROG})
	rm -f ${BT_DIR_BIN}/${BT_PROG}
.endif
.endif

# Define an emtpy target, if needed.
.if !target(_progdeinstall)
progdeinstall::
.endif

.if defined(BT_PROG) && !empty(BT_PROG)
real-tologic: prog-tologic
prog-tologic:
	@echo "logic() {" >> Logic.bt
	@echo "    bt_target ${BT_PROG}" >> Logic.bt
	@echo "}" >> Logic.bt
	@echo >> Logic.bt
	@echo "target_${BT_PROG}() {" >> Logic.bt
.if defined(QT_HDRS) && !empty(QT_HDRS)
	@echo "    BT_TYPE=qt_program" >> Logic.bt
	@echo "    QT_HEADERS=\"${QT_HDRS}\"" >> Logic.bt
.else
	@echo "    BT_TYPE=program" >> Logic.bt
.endif
	@echo "    BT_SOURCES=\"${BT_SRCS}\"" >> Logic.bt
.if defined(BT_FLAGS_CPP) && !empty(BT_FLAGS_CPP)
	@echo "    BT_FLAGS_CPP=\"$$BT_FLAGS_CPP ${BT_FLAGS_CPP:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_FLAGS_CC) && !empty(BT_FLAGS_CC)
	@echo "    BT_FLAGS_CC=\"$$BT_FLAGS_CC ${BT_FLAGS_CC:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_FLAGS_CXX) && !empty(BT_FLAGS_CXX)
	@echo "    BT_FLAGS_CXX=\"$$BT_FLAGS_CXX ${BT_FLAGS_CXX:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_FLAGS_LD) && !empty(BT_FLAGS_LD)
	@echo "    BT_FLAGS_LD=\"$$BT_FLAGS_LD ${BT_FLAGS_LD:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_LIBS) && !empty(BT_LIBS)
	@echo "    BT_LIBS=\"$$BT_LIBS ${BT_LIBS:S/"/\\\\"/g}\"" >> Logic.bt
.endif
.if defined(BT_MAN) && !empty(BT_MAN)
	@echo "    BT_MANPAGES=\"${BT_MAN}\"" >> Logic.bt
.endif
	@echo "}" >> Logic.bt
.endif

# Related stuff
.include <bt.man.mk>
.include <bt.dep.mk>
.include <bt.clean.mk>

${BT_TARGETS}:	# ensure existence
